@@ -92,6 +92,7 @@ zend_function_entry swfed_functions[] = {
     PHP_ME(swfed,  getActionData, NULL, 0)
     PHP_ME(swfed,  disasmActionData, NULL, 0)
     PHP_ME(swfed,  setActionVariables, NULL, 0)
+    PHP_ME(swfed,  replaceActionStrings, NULL, 0)
     PHP_ME(swfed,  replaceMovieClip, NULL, 0)
 
     PHP_ME(swfed,  setCompressLevel, NULL, 0)
@@ -104,7 +105,7 @@ zend_function_entry swfed_functions[] = {
 };
 /* }}} */
 
-#define SWFED_VERSION "0.40"
+#define SWFED_VERSION "0.41"
 
 /* {{{ swfed_module_entry
  */
@@ -474,6 +475,8 @@ PHP_METHOD(swfed, getTagDetail) {
         swf_tag_sprite_detail_t   *tag_sprite;
         swf_tag_shape_detail_t    *tag_shape;
         swf_tag_place_detail_t    *tag_place;
+        swf_action_t   *action;
+        int action_list_count;
 	zval *data = NULL;
       case 6:  // DefineBitsJPEG
       case 21: // DefineBitsJPEG2
@@ -517,8 +520,12 @@ PHP_METHOD(swfed, getTagDetail) {
         if (tag->code == 59) { // DoInitAction
             add_assoc_long(return_value, "action_sprite", tag_action->action_sprite);
         }
-        if (tag_action->action_record && tag_action->action_record_len) {
-            add_assoc_long(return_value, "action_record_len", tag_action->action_record_len);
+        action = tag_action->action_list->head;
+        if (action) {
+            for (action_list_count = 0 ; action ; action_list_count++) {
+                action = action->next;
+            }
+            add_assoc_long(return_value, "action_list_count", action_list_count);
         }
         break;
       case 37: // DefineEditText;
@@ -1208,7 +1215,7 @@ PHP_METHOD(swfed, replaceEditString) {
 PHP_METHOD(swfed, getActionData) {
     long tag_seqno = 0;
     swf_object_t *swf = NULL;
-    unsigned char *data_ref = NULL;
+    unsigned char *data = NULL;
     char *new_buff = NULL;
     unsigned long data_len = 0;
     
@@ -1216,17 +1223,19 @@ PHP_METHOD(swfed, getActionData) {
         RETURN_FALSE;
     }
     swf = get_swf_object(getThis() TSRMLS_CC);
-    data_ref = swf_object_get_actiondata(swf, &data_len, tag_seqno);
-    if (data_ref == NULL) {
+    data = swf_object_get_actiondata(swf, &data_len, tag_seqno);
+    if (data == NULL) {
         fprintf(stderr, "getActionData: Can't get_actiondata\n");
         RETURN_FALSE;
     }
     new_buff = emalloc(data_len);
     if (new_buff == NULL) {
         fprintf(stderr, "getActionData: Can't emalloc new_buff\n");
+        free(data);
         RETURN_FALSE;
     }
-    memcpy(new_buff, data_ref, data_len);
+    memcpy(new_buff, data, data_len);
+    free(data);
     RETURN_STRINGL(new_buff, data_len, 0);
 }
 
@@ -1242,7 +1251,8 @@ PHP_METHOD(swfed, disasmActionData) {
     array_init(return_value);
     bs = bitstream_open();
     bitstream_input(bs, (unsigned char*) data, data_len);
-    action_list = swf_action_list_create(bs);
+    action_list = swf_action_list_create();
+    swf_action_list_parse(bs, action_list);
     bitstream_close(bs);
     if (action_list) {
         swf_action_t *action = action_list->head;
@@ -1299,6 +1309,49 @@ PHP_METHOD(swfed, setActionVariables) {
     RETURN_TRUE;
 }
 
+PHP_METHOD(swfed, replaceActionStrings) {
+    zval *zid, *arr, **entry;
+    HashTable *arr_hash;
+    HashPosition    pos;
+    char            *str_key, *str_value;
+    uint            str_key_len, str_value_len;
+    ulong tmp;
+    char tmp_str[17];
+    int ret;
+    y_keyvalue_t *kv;
+    swf_object_t *swf = get_swf_object(getThis() TSRMLS_CC);
+    
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a", &arr) == FAILURE) {
+        RETURN_FALSE;
+    }
+    kv = y_keyvalue_open();
+
+    arr_hash = Z_ARRVAL_P(arr);
+    zend_hash_internal_pointer_reset_ex(Z_ARRVAL_P(arr), &pos);
+    while (zend_hash_get_current_data_ex(Z_ARRVAL_P(arr), (void **)&entry, &
+                                         pos) == SUCCESS) {
+        str_value = Z_STRVAL_PP(entry);
+        str_value_len = Z_STRLEN_PP(entry);
+        ret = zend_hash_get_current_key_ex(Z_ARRVAL_P(arr), &str_key, &
+                                           str_key_len, &tmp, 0, &pos);
+        switch (ret) {
+        case HASH_KEY_IS_STRING:
+            y_keyvalue_set(kv, str_key, str_key_len - 1, str_value, str_value_len);
+            break;
+        case HASH_KEY_IS_LONG:
+            snprintf(tmp_str, 17, "%ld\0", tmp);
+            y_keyvalue_set(kv, tmp_str, strlen(tmp_str), str_value, str_value_len);
+            break;
+        default:
+            php_error_docref(NULL TSRMLS_CC, E_WARNING, "Array keys invalid type(%d).", ret);
+        }
+        zend_hash_move_forward_ex(Z_ARRVAL_P(arr), &pos);
+    }
+    swf_object_replace_action_strings(swf, kv);
+    y_keyvalue_close(kv);
+    RETURN_TRUE;
+}
+
 PHP_METHOD(swfed, replaceMovieClip) {
     char *instance_name = NULL, *swf_data = NULL;
     int  instance_name_len = 0, swf_data_len = 0;
