@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2006 The PHP Group                                |
+  | Copyright (c) 1997-2008 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -12,11 +12,11 @@
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
-  | Author: yoya@awm.jp                                                  |
+  | Author: Yoshihiro Yamazaki <yoya@awm.jp>                             |
   +----------------------------------------------------------------------+
 */
 
-/* $Id: header,v 1.16.2.1 2006/01/01 12:50:00 sniper Exp $ */
+/* $Id:$ */
 
 #ifndef PHP_SWFED_H
 #define PHP_SWFED_H
@@ -47,6 +47,7 @@ static zend_class_entry *swfeditor_ce;
 PHP_METHOD(swfed, __construct);
 PHP_METHOD(swfed, input);
 PHP_METHOD(swfed, output);
+PHP_METHOD(swfed, swfInfo);
 PHP_METHOD(swfed, _destroy_and_exit); // for debug
 //
 PHP_METHOD(swfed, getHeaderInfo);
@@ -56,8 +57,12 @@ PHP_METHOD(swfed, getTagDetail);
 PHP_METHOD(swfed, getTagInfo);
 PHP_METHOD(swfed, getTagData);
 PHP_METHOD(swfed, replaceTagData);
+PHP_METHOD(swfed, getTagDataByCID);
+PHP_METHOD(swfed, replaceTagDataByCID);
 PHP_METHOD(swfed, getTagContentsByCID);
 PHP_METHOD(swfed, replaceTagContentsByCID);
+PHP_METHOD(swfed, removeTag);
+PHP_METHOD(swfed, printTagData);
 //
 PHP_METHOD(swfed, getShapeData);
 PHP_METHOD(swfed, replaceShapeData);
@@ -80,11 +85,15 @@ PHP_METHOD(swfed, replaceEditString);
 PHP_METHOD(swfed, getActionData);
 PHP_METHOD(swfed, disasmActionData);
 PHP_METHOD(swfed, setActionVariables);
+PHP_METHOD(swfed, replaceActionStrings);
 PHP_METHOD(swfed, replaceMovieClip);
-PHP_METHOD(swfed, swfInfo);
+
 PHP_METHOD(swfed, setCompressLevel);
 PHP_METHOD(swfed, rebuild);
+PHP_METHOD(swfed, purgeUselessContents);
 
+PHP_METHOD(swfed, isShapeTagData);
+PHP_METHOD(swfed, isBitmapTagData);
 /* 
   	Declare any global variables you may need between the BEGIN
 	and END macros here:     
