@@ -11,11 +11,12 @@
 #include "swf_header.h"
 #include "swf_header.h"
 #include "swf_tag.h"
+#include "swf_tag_sprite.h"
 
 typedef struct swf_object_ {
     swf_header_t header;
     swf_header_movie_t header_movie;
-    swf_tag_t *tag;
+    swf_tag_t *tag_head, *tag_tail;
     unsigned shape_adjust_mode;
     unsigned compress_level;
 } swf_object_t;
@@ -29,25 +30,50 @@ extern swf_object_t *swf_object_open(void);
 extern void swf_object_close(swf_object_t *swf);
 extern int swf_object_input(swf_object_t *swf, unsigned char *data, unsigned long data_len);
 extern unsigned char *swf_object_output(swf_object_t *swf, unsigned long *length);
+extern void swf_object_print(swf_object_t *swf);
+
+/* --- */
+
+extern int swf_object_rebuild(swf_object_t *swf);
+extern void swf_object_purge_contents(swf_object_t *swf);
 
 /* --- */
 
-extern void swf_object_rebuild(swf_object_t *swf);
-extern void swf_object_print(swf_object_t *swf);
 extern unsigned char *swf_object_get_tagdata(swf_object_t *swf, int tag_seqno,
                                              unsigned long *length);
 extern int swf_object_replace_tagdata(swf_object_t *swf, int tag_seqno,
                                       unsigned char *data,
                                       unsigned long length);
+extern unsigned char *swf_object_get_tagdata_bycid(swf_object_t *swf,
+						   int cid,
+						   unsigned long *length);
+extern int swf_object_replace_tagdata_bycid(swf_object_t *swf,
+					    int cid,
+					    unsigned char *data,
+					    unsigned long length);
 extern unsigned char *swf_object_get_tagcontents_bycid(swf_object_t *swf,
                                                        int cid,
                                                        unsigned long *length);
 extern int swf_object_replace_tagcontents_bycid(swf_object_t *swf, int cid,
                                                 unsigned char *data,
                                                 unsigned long length);
+extern int swf_object_remove_tag(swf_object_t *swf, int tag_seqno,
+                                 int tag_seqno_in_sprite);
+extern int swf_object_print_tagdata(swf_object_t *swf, 
+                                    unsigned char *data, unsigned long length);
+
 
 /* --- */
 
+extern swf_tag_t *swf_object_search_tag_byseqno(swf_object_t *swf,
+						int tag_seqno);
+extern swf_tag_t *swf_object_search_tag_in_sprite_byseqno(swf_tag_sprite_detail_t *sprite, int tag_seqno);
+
+extern swf_tag_t *swf_object_search_tag_bycid(swf_object_t *swf,
+						int cid);
+extern swf_tag_t *swf_object_search_shape_tag(swf_object_t *swf,
+                                               int shape_id);
+
 extern swf_tag_t *swf_object_search_bitmap_tag(swf_object_t *swf,
                                                int bitmap_id);
 
@@ -99,12 +125,14 @@ extern unsigned char *swf_object_get_actiondata(swf_object_t *swf, unsigned long
 extern int swf_object_insert_action_setvariables(swf_object_t *swf,
                                                  y_keyvalue_t *kv);
 
+extern int swf_object_replace_action_strings(swf_object_t *swf,
+                                             y_keyvalue_t *kv);
+
 extern int swf_object_replace_movieclip(swf_object_t *swf,
                                         unsigned char *instance_name,
                                         int instancee_name_len,
                                         unsigned char *swf_data,
-                                        int swf_data_len,
-                                        int unused_cid_purge);
+                                        int swf_data_len);
 
 extern int swf_object_apply_shapematrix_factor(swf_object_t *swf,int shape_id,
                                                double scale_x, double scale_y,
@@ -117,4 +145,14 @@ extern int swf_object_apply_shaperect_factor(swf_object_t *swf,int shape_id,
                                              signed int trans_y);
 extern int swf_object_apply_shapetype_tilled(swf_object_t *swf,int shape_id);
 
+
+/* --- */
+
+extern int swf_object_is_shape_tagdata(unsigned char *data, int data_len);
+
+extern int swf_object_is_bitmap_tagdata(unsigned char *data, int data_len);
+extern int swf_object_replace_tag(struct swf_object_ *swf, 
+				  swf_tag_t *old_tag, swf_tag_t *new_tag);
+
+
 #endif /* __SWF_OBJECT_H__ */
