@@ -30,7 +30,7 @@ swf_tag_info_t swf_tag_info_table[] = {
     { 7, "DefineButton", NULL},
     { 8, "JPEGTables", swf_tag_jpegt_detail_handler },
     { 9, "SetBackgroundColor", NULL },
-    { 10, "DefineFont ", NULL},
+    { 10, "DefineFont", NULL},
     { 11, "DefineText", NULL },
     { 12, "DoAction", swf_tag_action_detail_handler },
     { 13, "DefineFontInfo", NULL },
@@ -43,6 +43,7 @@ swf_tag_info_t swf_tag_info_table[] = {
     { 20, "DefineBitsLossless", swf_tag_lossless_detail_handler },
     { 21, "DefineBitsJPEG2", swf_tag_jpeg_detail_handler },
     { 22, "DefineShape2", swf_tag_shape_detail_handler },
+    { 24, "Protect", NULL },
     { 26, "PlaceObject2", swf_tag_place_detail_handler },
     { 28, "RemoveObject2", NULL },
     { 32, "DefineShape3", swf_tag_shape_detail_handler },
@@ -56,13 +57,13 @@ swf_tag_info_t swf_tag_info_table[] = {
     { 46, "DefineMorphShape", swf_tag_shape_detail_handler },
     { 48, "DefineFont2", NULL } ,
     { 56, "Export", NULL } ,
-    { 59, "DoInitAction", NULL } ,
+    { 59, "DoInitAction", swf_tag_action_detail_handler },
     { 69, "FileAttributes", NULL },
     { 73, "DefineFontAlignZones", NULL },
     { 74, "CSMTextSettings", NULL },
     { 75, "DefineFont3", NULL } ,
-    { 83, "DefineShape4", NULL /*swf_tag_shape_detail_handler*/ },
-    { 84, "DefineMorphShape2", NULL /*swf_tag_shape_detail_handler*/ },
+    { 83, "DefineShape4",  swf_tag_shape_cid_handler },
+    { 84, "DefineMorphShape2", swf_tag_shape_cid_handler },
     { 88, "DefineFontName", NULL } ,
     { 777,"Reflex", NULL } ,
 };
@@ -90,7 +91,7 @@ swf_tag_t *swf_tag_create(bitstream_t *bs) {
         return NULL;
     }
     tag = calloc(sizeof(*tag), 1);
-    tag->tag = tag_and_length >> 6;
+    tag->code = tag_and_length >> 6;
     tag->length = tag_and_length & 0x3f;
     tag->length_longformat = 0;
     if (tag->length == 0x3f) {
@@ -137,7 +138,7 @@ static int swf_tag_and_length_build(bitstream_t *bs, swf_tag_t *tag) {
     if (tag->length >= 0x3f) {
         tag->length_longformat = 1;
     } else {
-        switch (tag->tag) {
+        switch (tag->code) {
           case 6:  // DefineBitsJPEG
           case 21: // DefineBitsJPEG2
           case 35: // DefineBitsJPEG3
@@ -152,11 +153,11 @@ static int swf_tag_and_length_build(bitstream_t *bs, swf_tag_t *tag) {
         }
     }
     if (tag->length_longformat) {
-        tag_and_length = (tag->tag << 6) | 0x3f;
+        tag_and_length = (tag->code << 6) | 0x3f;
         bitstream_putbytesLE(bs, tag_and_length, 2);
         bitstream_putbytesLE(bs, tag->length, 4);
     } else {
-        tag_and_length = (tag->tag << 6) | tag->length;
+        tag_and_length = (tag->code << 6) | tag->length;
         bitstream_putbytesLE(bs, tag_and_length, 2);
     }
     return 0;
@@ -175,21 +176,21 @@ int swf_tag_build(bitstream_t *bs, swf_tag_t *tag, struct swf_object_ *swf) {
         fprintf(stderr, "swf_tag_build: tag == NULL\n");
         return 1;
     }
-//    fprintf(stderr, "XXX: swf_tag_build: tag->tag=%d\n",tag->tag);
+//    fprintf(stderr, "XXX: swf_tag_build: tag->code=%d\n",tag->code);
     if (tag->data) {
         swf_tag_and_length_build(bs, tag);
         bitstream_putstring(bs, tag->data, tag->length);
     } else if (tag->detail) {
-        tag_info = get_swf_tag_info(tag->tag);
+        tag_info = get_swf_tag_info(tag->code);
         if ((tag_info == NULL) || (tag_info->detail_handler == NULL)) {
-            fprintf(stderr, "swf_tag_build: not implemented yet. detail build tag->tag=%d\n",
-                tag->tag);
+            fprintf(stderr, "swf_tag_build: not implemented yet. detail build tag->code=%d\n",
+                tag->code);
             return 1;
         }
         detail_handler = tag_info->detail_handler();
         if (detail_handler->output == NULL) {
-            fprintf(stderr, "swf_tag_build: detail_handler->output == NULL: tag->tag=%d\n",
-                    tag->tag);
+            fprintf(stderr, "swf_tag_build: detail_handler->output == NULL: tag->code=%d\n",
+                    tag->code);
             return 1;
         }
         data = detail_handler->output(tag, &data_len, swf);
@@ -212,21 +213,21 @@ int
 swf_tag_rebuild(swf_tag_t *tag, struct swf_object_ *swf) {
     swf_tag_info_t *tag_info = NULL;
     swf_tag_detail_handler_t * detail_handler = NULL;
-    int ret;
-    tag_info = get_swf_tag_info(tag->tag);
-    if (tag_info == NULL) {
-        return 1; // no info
+    void *detail;
+    tag_info = get_swf_tag_info(tag->code);
+    if ((tag_info == NULL) || (tag_info->detail_handler == NULL)) {
+        return 0; // no info
     }
     detail_handler = tag_info->detail_handler();
     if (detail_handler == NULL) {
-        return 1; // no detail handler
+        return 0; // no detail handler
     }
     if ((detail_handler->input == NULL) || (detail_handler->output == NULL)) {
-        return 1; // no input or output handler
+        return 0; // no input or output handler
     }
-    ret = detail_handler->input(tag, swf);
-    if (ret) {
-        fprintf(stderr, "swf_tag_rebuild: detail_hander->input failed tag_no=%d\n", tag->tag);
+    detail = swf_tag_create_input_detail(tag, swf);
+    if (detail == NULL) {
+        fprintf(stderr, "swf_tag_rebuild: swf_tag_create_input_detail failed tag->code=%d\n", tag->code);
         return 1;
     }
     free(tag->data);
@@ -243,10 +244,10 @@ swf_tag_print(swf_tag_t *tag, struct swf_object_ *swf, int indent_depth) {
         fprintf(stderr, "swf_tag_print: tag == NULL\n");
         return ;
     }
-    tag_info = get_swf_tag_info(tag->tag);
+    tag_info = get_swf_tag_info(tag->code);
     tag_name = (tag_info)?tag_info->name:"Unknown";
 //  print_indent(indent_depth);
-    printf("tag=%s(%d)", tag_name, tag->tag);
+    printf("tag=%s(%d)", tag_name, tag->code);
     if (tag->length > 0) {
         printf("  length=%lu",  tag->length);
     }
@@ -272,25 +273,30 @@ void *swf_tag_create_input_detail(swf_tag_t *tag, struct swf_object_ *swf) {
     if (tag->detail) {
         return tag->detail;
     }
-    tag_info = get_swf_tag_info(tag->tag);
+    tag_info = get_swf_tag_info(tag->code);
     if (tag_info && tag_info->detail_handler) {
         int result;
         swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
         if (detail_handler->create == NULL) {
-            fprintf(stderr, "detail_handler->create == NULL (tag=%d)\n",
-                    tag->tag);
+            fprintf(stderr, "swf_tag_create_input_detail: detail_handler->create == NULL (tag=%d)\n",
+                    tag->code);
             return NULL;
         }
         tag->detail = detail_handler->create();
         if (tag->detail == NULL) {
-            fprintf(stderr, "can't create tag detail (tag=%d)\n", tag->tag);
+            fprintf(stderr, "swf_tag_create_input_detail: can't create tag detail (tag=%d)\n", tag->code);
             return NULL;
         }
         result = detail_handler->input(tag, swf);
         if (result) {
-            fprintf(stderr, "can't input tag detail (result=%d)\n", result);
+            fprintf(stderr, "swf_tag_create_input_detail: can't input tag detail (result=%d)\n", result);
             return NULL;
         }
+    } else {
+        fprintf(stderr, "swf_tag_create_input_detail: tag_info or detail_handler  == NULL\n");
+    }
+    if (tag->detail == NULL) {
+        fprintf(stderr, "swf_tag_create_input_detail: function tail but tag->detail == NULL (tag->code=%d)\n", tag->code);
     }
     return tag->detail;
 }
@@ -300,14 +306,14 @@ void swf_tag_destroy_detail(swf_tag_t *tag) {
         return;
     }
     if (tag->detail) {
-        swf_tag_info_t *tag_info = get_swf_tag_info(tag->tag);
+        swf_tag_info_t *tag_info = get_swf_tag_info(tag->code);
         if (tag_info && tag_info->detail_handler) {
             swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
             if (detail_handler->destroy) {
                 detail_handler->destroy(tag);
             } else {
 	        fprintf(stderr, "detail_handler->destroy == NULL (tag=%d)\n",
-                        tag->tag);
+                        tag->code);
             }
         } else {
 	    fprintf(stderr, "not impremented yet. destroy tag detail\n");
@@ -325,7 +331,7 @@ swf_tag_get_cid(swf_tag_t *tag) {
         fprintf(stderr, "swf_tag_get_cid: tag == NULL\n");
         return 1;
     }
-    tag_no = tag->tag;
+    tag_no = tag->code;
     tag_info = get_swf_tag_info(tag_no);
     if (tag_info && tag_info->detail_handler) {
         swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
@@ -342,7 +348,7 @@ swf_tag_get_cid(swf_tag_t *tag) {
           case 17: // DefineButtonSound
           case 33: // DefineText2
           case 34: // DefineButton2
-          case 39: // DefineSprite ///////////////
+          case 39: // DefineSprite
           case 46: // DefineMorphShape
           case 48: // DefineFont2
           case 88: // DefineFontName
@@ -365,7 +371,7 @@ swf_tag_replace_cid(swf_tag_t *tag, int cid) {
         fprintf(stderr, "swf_tag_replace_cid: tag == NULL\n");
         return 1; // failure
     }
-    tag_no = tag->tag;
+    tag_no = tag->code;
     tag_info = get_swf_tag_info(tag_no);
     if (tag_info && tag_info->detail_handler) {
         swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
@@ -382,8 +388,8 @@ swf_tag_replace_cid(swf_tag_t *tag, int cid) {
           case 17: // DefineButtonSound
           case 33: // DefineText2
           case 34: // DefineButton2
-          case 39: // DefineSprite ///////////////
-          case 46:  // DefineMorphShape
+          case 39: // DefineSprite
+          case 46: // DefineMorphShape
           case 48: // DefineFont2
           case 88: // DefineFontName
             if (tag->data) {
@@ -404,7 +410,7 @@ swf_tag_get_refcid(swf_tag_t *tag) {
         fprintf(stderr, "swf_tag_get_refcid: tag == NULL\n");
         return -1;
     }
-    if (isPlaceTag(tag->tag)) { // PlaceObject, PlaceObject2
+    if (isPlaceTag(tag->code)) { // PlaceObject, PlaceObject2
         swf_tag_place_detail_t *swf_tag_place;
         swf_tag_place = swf_tag_create_input_detail(tag, NULL);
         if (swf_tag_place == NULL) {
@@ -422,7 +428,7 @@ swf_tag_replace_refcid(swf_tag_t *tag, int cid) {
         fprintf(stderr, "swf_tag_replace_refcid: tag == NULL\n");
         return 1; // failure
     }
-    if (isPlaceTag(tag->tag)) { // PlaceObject, PlaceObject2
+    if (isPlaceTag(tag->code)) { // PlaceObject, PlaceObject2
         swf_tag_place_detail_t *swf_tag_place;
         swf_tag_place = swf_tag_create_input_detail(tag, NULL);
         if (swf_tag_place == NULL) {
@@ -456,11 +462,11 @@ swf_tag_get_bitmap_size(swf_tag_t *tag,
         fprintf(stderr, "swf_tag_get_bitmap_size: swf_tag_create_input_detail failed\n");
         return 1;
     }
-    if (isBitsJPEGTag(tag->tag)) {
+    if (isBitsJPEGTag(tag->code)) {
         swf_tag_jpeg_detail_t *swf_tag_jpeg = (swf_tag_jpeg_detail_t *) tag->detail;
         ret = jpeg_size(swf_tag_jpeg->jpeg_data, swf_tag_jpeg->jpeg_data_len,
                   width, height);
-    } else if (isBitsLosslessTag(tag->tag)) {
+    } else if (isBitsLosslessTag(tag->code)) {
         swf_tag_lossless_detail_t *swf_tag_lossless = (swf_tag_lossless_detail_t *) tag->detail;
         *width  = swf_tag_lossless->width;
         *height = swf_tag_lossless->height;
@@ -482,8 +488,8 @@ swf_tag_get_jpeg_data(swf_tag_t *tag, unsigned long *length, int image_id, swf_t
         fprintf(stderr, "swf_tag_get_jpeg_data: length == NULL\n");
         return NULL;
     }
-    tag_info = get_swf_tag_info(tag->tag);
-    if ((tag->tag != 6) && (tag->tag != 21) && (tag->tag != 35)) {
+    tag_info = get_swf_tag_info(tag->code);
+    if ((tag->code != 6) && (tag->code != 21) && (tag->code != 35)) {
         return NULL;
     }
     if (swf_tag_create_input_detail(tag, NULL) == NULL) {
@@ -511,7 +517,7 @@ swf_tag_get_alpha_data(swf_tag_t *tag, unsigned long *length, int image_id) {
         fprintf(stderr, "swf_tag_get_alpha_data: length == NULL\n");
         return NULL;
     }
-    if (tag->tag != 35) { // ! DefineBitsJPEG3
+    if (tag->code != 35) { // ! DefineBitsJPEG3
         return NULL;
     }
     if (swf_tag_get_cid(tag) != image_id) {
@@ -541,26 +547,26 @@ swf_tag_replace_jpeg_data(swf_tag_t *tag, int image_id,
         fprintf(stderr, "swf_tag_replace_jpeg_data: jpeg_data == NULL\n");
         return 1;
     }
-    if (! isBitmapTag(tag->tag)) {
+    if (! isBitmapTag(tag->code)) {
         return 1;
     }
-    tag_info = get_swf_tag_info(tag->tag); // Bitmap Tag
+    tag_info = get_swf_tag_info(tag->code); // Bitmap Tag
     detail_handler = tag_info->detail_handler();
     if (detail_handler->get_cid(tag) !=  image_id) {
         return 1;
     }
 
-    if (tag->detail && (! isBitsJPEGTag(tag->tag))) {
+    if (tag->detail && (! isBitsJPEGTag(tag->code))) {
         detail_handler->destroy(tag);
         tag->detail = NULL;
     }
     if (alpha_data && (alpha_data_len > 0)) {
-        tag->tag = 35; // DefineBitsJPEG3
+        tag->code = 35; // DefineBitsJPEG3
     } else {
-        tag->tag = 21; // DefineBitsJPEG2
+        tag->code = 21; // DefineBitsJPEG2
     }
     
-    tag_info = get_swf_tag_info(tag->tag); // JPEG Tag
+    tag_info = get_swf_tag_info(tag->code); // JPEG Tag
     detail_handler = tag_info->detail_handler();
     if (tag->detail == NULL) {
         tag->detail = detail_handler->create();
@@ -595,8 +601,8 @@ swf_tag_get_png_data(swf_tag_t *tag, unsigned long *length, int image_id) {
         fprintf(stderr, "swf_tag_get_png_data: length == NULL\n");
         return NULL;
     }
-    tag_info = get_swf_tag_info(tag->tag);
-    if ((tag->tag != 20) && (tag->tag != 36)) {
+    tag_info = get_swf_tag_info(tag->code);
+    if ((tag->code != 20) && (tag->code != 36)) {
         return NULL;
     }
     if (swf_tag_create_input_detail(tag, NULL) == NULL) {
@@ -621,26 +627,26 @@ swf_tag_replace_png_data(swf_tag_t *tag, int image_id,
         fprintf(stderr, "swf_tag_replace_png_data: png_data == NULL\n");
         return 1;
     }
-    if (! isBitmapTag(tag->tag)) {
+    if (! isBitmapTag(tag->code)) {
         return 1;
     }
-    tag_info = get_swf_tag_info(tag->tag); // Bitmap Tag
+    tag_info = get_swf_tag_info(tag->code); // Bitmap Tag
     detail_handler = tag_info->detail_handler();
     if (detail_handler->get_cid(tag) != image_id) {
         return 1;
     }
     
-    if (tag->detail && (! isBitsLosslessTag(tag->tag))) {
+    if (tag->detail && (! isBitsLosslessTag(tag->code))) {
         detail_handler->destroy(tag);
         tag->detail = NULL;
     }
-    if (tag->tag == 20) {
-        tag->tag = 20; // DefineBitsLossless
+    if (tag->code == 20) {
+        tag->code = 20; // DefineBitsLossless
     } else {
-        tag->tag = 36; // DefineBitsLossless2
+        tag->code = 36; // DefineBitsLossless2
     }
     
-    tag_info = get_swf_tag_info(tag->tag); // Lossless Tag
+    tag_info = get_swf_tag_info(tag->code); // Lossless Tag
     detail_handler = tag_info->detail_handler();
     if (tag->detail == NULL) {
         tag->detail = detail_handler->create();
@@ -678,26 +684,26 @@ swf_tag_replace_gif_data(swf_tag_t *tag, int image_id,
         fprintf(stderr, "swf_tag_replace_gif_data: gif_data == NULL\n");
         return 1;
     }
-    if (! isBitmapTag(tag->tag)) {
+    if (! isBitmapTag(tag->code)) {
         return 1;
     }
-    tag_info = get_swf_tag_info(tag->tag); // Bitmap Tag
+    tag_info = get_swf_tag_info(tag->code); // Bitmap Tag
     detail_handler = tag_info->detail_handler();
     if (detail_handler->get_cid(tag) != image_id) {
         return 1;
     }
     
-    if (tag->detail && (! isBitsLosslessTag(tag->tag))) {
+    if (tag->detail && (! isBitsLosslessTag(tag->code))) {
         detail_handler->destroy(tag);
         tag->detail = NULL;
     }
-    if (tag->tag == 20) {
-        tag->tag = 20; // DefineBitsLossless
+    if (tag->code == 20) {
+        tag->code = 20; // DefineBitsLossless
     } else {
-        tag->tag = 36; // DefineBitsLossless2
+        tag->code = 36; // DefineBitsLossless2
     }
     
-    tag_info = get_swf_tag_info(tag->tag); // Lossless Tag
+    tag_info = get_swf_tag_info(tag->code); // Lossless Tag
     detail_handler = tag_info->detail_handler();
     if (tag->detail == NULL) {
         tag->detail = detail_handler->create();
@@ -734,8 +740,8 @@ swf_tag_get_sound_data(swf_tag_t *tag, unsigned long *length, int sound_id) {
         fprintf(stderr, "swf_tag_get_sound_data: length == NULL\n");
         return NULL;
     }
-    tag_info = get_swf_tag_info(tag->tag);
-    if (tag->tag != 14) { // DefineSound
+    tag_info = get_swf_tag_info(tag->code);
+    if (tag->code != 14) { // DefineSound
         return NULL;
     }
     if (! tag->detail) {
@@ -761,7 +767,7 @@ swf_tag_replace_melo_data(swf_tag_t *tag, int sound_id,
         fprintf(stderr, "swf_tag_replace_melo_data: melo_data == NULL\n");
         return 1;
     }
-    if (tag->tag != 14) { // DefineSound
+    if (tag->code != 14) { // DefineSound
         return 1;
     }
     if (swf_tag_get_cid(tag) != sound_id) {
@@ -793,7 +799,7 @@ swf_tag_get_edit_string(swf_tag_t *tag,
         fprintf(stderr, "swf_tag_get_edit_string: variable_name == NULL\n");
         return NULL;
     }
-    if (tag->tag != 37) { // DefineEditText
+    if (tag->code != 37) { // DefineEditText
         return NULL;
     }
     if (swf_tag_create_input_detail(tag, swf) == NULL) {
@@ -818,7 +824,7 @@ swf_tag_replace_edit_string(swf_tag_t *tag,
         fprintf(stderr, "swf_tag_replace_edit_string: variable_name == NULL\n");
         return 1;
     }
-    if (tag->tag != 37) { // DefineEditText
+    if (tag->code != 37) { // DefineEditText
         return 1;
     }
     if (swf_tag_create_input_detail(tag, swf) == NULL) {
@@ -847,8 +853,8 @@ swf_tag_apply_shape_matrix_factor(swf_tag_t *tag, int shape_id,
         fprintf(stderr, "swf_tag_apply_shape_matrix_factor: tag == NULL\n");
         return 1;
     }
-    if ((tag->tag != 2) && (tag->tag != 22) && (tag->tag !=32 )
-        && (tag->tag != 46)) {
+    if ((tag->code != 2) && (tag->code != 22) && (tag->code !=32 )
+        && (tag->code != 46)) {
         // ! DefineShape1,2,3, DefineMorphShape
         return 1;
     }
@@ -881,8 +887,8 @@ swf_tag_apply_shape_rect_factor(swf_tag_t *tag, int shape_id,
         fprintf(stderr, "swf_tag_apply_shape_rect_factor: tag == NULL\n");
         return 1;
     }
-    if ((tag->tag != 2) && (tag->tag != 22) && (tag->tag !=32 )
-        && (tag->tag != 46)) {
+    if ((tag->code != 2) && (tag->code != 22) && (tag->code !=32 )
+        && (tag->code != 46)) {
         // ! DefineShape1,2,3, DefineMorphShape
         return 1;
     }
@@ -912,8 +918,8 @@ swf_tag_apply_shape_type_tilled(swf_tag_t *tag, int shape_id,
         fprintf(stderr, "swf_tag_apply_shape_type_tylled: tag == NULL\n");
         return 1;
     }
-    if ((tag->tag != 2) && (tag->tag != 22) && (tag->tag !=32 )
-        && (tag->tag != 46)) {
+    if ((tag->code != 2) && (tag->code != 22) && (tag->code !=32 )
+        && (tag->code != 46)) {
         // ! DefineShape1,2,3, DefineMorphShape
         return 1;
     }
@@ -937,16 +943,25 @@ swf_tag_t *
 swf_tag_create_action_setvariables(y_keyvalue_t *kv) {
     int ret;
     swf_tag_t *tag = NULL;
+    swf_tag_action_detail_t *swf_tag_action;
     if (kv == NULL) {
         fprintf(stderr, "swf_tag_create_action_setvariables: kv == NULL\n");
         return NULL;
     }
     tag = calloc(sizeof(*tag), 1);
-    tag->tag = 12; // DoAction
-    swf_tag_info_t *tag_info = get_swf_tag_info(tag->tag);
+    tag->code = 12; // DoAction
+    swf_tag_info_t *tag_info = get_swf_tag_info(tag->code);
     swf_tag_detail_handler_t *detail_handler = tag_info->detail_handler();
     tag->detail = detail_handler->create();
-    ret = swf_tag_action_create_setvaribles(tag, kv);
+    swf_tag_action = (swf_tag_action_detail_t*) tag->detail;
+    swf_tag_action->action_list = swf_action_list_create();
+    if (swf_tag_action->action_list == NULL) {
+        fprintf(stderr, "swf_tag_create_action_setvariables: swf_action_list_create failed\n");
+        swf_tag_destroy(tag);
+        return NULL;
+    }
+    swf_action_list_append_top(swf_tag_action->action_list, 0, NULL, 0); // End Action
+    ret = swf_tag_action_top_append_varibles(tag, kv);
     if (ret) {
         swf_tag_destroy(tag);
         return NULL;
@@ -970,7 +985,7 @@ swf_tag_put_action_setvariables(swf_tag_t *tag, y_keyvalue_t *kv,
         fprintf(stderr, "swf_tag_put_action_setvariables: swf_tag_create_input_detail failed\n");
         return 1;
     }
-    ret = swf_tag_action_put_setvaribles(tag, kv);
+    ret = swf_tag_action_top_append_varibles(tag, kv);
     if (ret) {
         swf_tag_destroy(tag);
         return 1; // NG
@@ -982,6 +997,35 @@ swf_tag_put_action_setvariables(swf_tag_t *tag, y_keyvalue_t *kv,
     return 0;
 }
 
+int
+swf_tag_replace_action_strings(swf_tag_t *tag, y_keyvalue_t *kv,
+                               int *modified, struct swf_object_ *swf) {
+    int ret;
+    if (tag == NULL) {
+        fprintf(stderr, "swf_tag_replace_action_string: tag == NULL\n");
+        return 1; // NG
+    }
+    if (kv == NULL) {
+        fprintf(stderr, "swf_tag_replace_action_string: kv == NULL\n");
+        return 1; // NG
+    }
+    if (swf == NULL) {
+        fprintf(stderr, "swf_tag_replace_action_string: swf == NULL\n");
+        return 1; // NG
+    }
+    if (swf_tag_create_input_detail(tag, swf) == NULL) {
+        fprintf(stderr, "swf_tag_replace_action_string: swf_tag_create_input_detail failed\n");
+        return 1; // NG
+    }
+    ret = swf_tag_action_replace_strings(tag, kv, modified);
+    if (ret) {
+      fprintf(stderr, "swf_tag_replace_action_string: swf_tag_action_replace_strings failed\n");
+    }
+    return ret;
+}
+
+
+// タグの中味を移し替える
 swf_tag_t *
 swf_tag_move(swf_tag_t *from_tag) {
     swf_tag_t *to_tag = NULL;
@@ -990,7 +1034,7 @@ swf_tag_move(swf_tag_t *from_tag) {
         return NULL;
     }
     to_tag = calloc(sizeof(*to_tag), 1);
-    to_tag->tag = from_tag->tag;
+    to_tag->code = from_tag->code;
     to_tag->length = from_tag->length;
     to_tag->length_longformat = from_tag->length_longformat;
     to_tag->data = from_tag->data;
